#ifndef _UBOOT_SB21_H
#define _UBOOT_SB21_H

#ifdef	__cplusplus
extern "C"
{
#endif

#if defined(CONFIG_CPU_EXYNOS3475)
#define CONFIG_SECURE_BOOT_V30
#elif defined(CONFIG_CPU_EXYNOS5430) || defined(CONFIG_CPU_EXYNOS5412) || defined(CONFIG_CPU_EXYNOS5260) || defined(CONFIG_CPU_EXYNOS4415) || defined(CONFIG_CPU_EXYNOS3250) || defined(CONFIG_CPU_EXYNOS3472)
#define CONFIG_SECURE_BOOT_V24
#elif defined(CONFIG_SMDK5410)
#define CONFIG_SECURE_BOOT_V23
#else
#define CONFIG_SECURE_BOOT_V20
#endif

////////////////////////////////////////////////////////////////////////
// SecureBoot return value define
#define		SB_OK					0x00000000
#define		SB_OFF					0x80000000

//------------------------------------------------------------------------
#define		SB_ERROR_VALIDATE_PUBLIC_KEY_INFO	0xFFF10000
#define		SB_ERROR_VERIFY_PSS_RSA_SIGNATURE	0xFFF20000
#define		SB_ERROR_CHECK_INTEGRITY_CODE		0xFFF30000

//------------------------------------------------------------------------
// added for Secure Boot 2.0
#define		SB_ERROR_GENERATE_PSS_RSA_SIGNATURE	0xFFF40000
#define		SB_ERROR_GENERATE_PUBLIC_KEY_INFO	0xFFF50000
#define		SB_ERROR_GENERATE_SB_CONTEXT		0xFFF60000
#define		SB_ERROR_ENCRYPTION			0xFFF70000

#define		SB_ERROR_AES_PARM			0x0000A000
#define		SB_ERROR_AES_SET_ALGO			0x0000B000
#define		SB_ERROR_AES_ENCRYPT			0x0000C000
#define		SB_ERROR_AES_DECRYPT			0x0000D000

//------------------------------------------------------------------------
#define		SB_ERROR_HMAC_SHA1_SET_INFO		0x00000010
#define		SB_ERROR_HMAC_SHA1_INIT			0x00000020
#define		SB_ERROR_HMAC_SHA1_UPDATE		0x00000030
#define		SB_ERROR_HMAC_SHA1_FINAL		0x00000040
#define		SB_ERROR_MEM_CMP			0x00000050
#define		SB_ERROR_SHA1_INIT			0x00000060
#define		SB_ERROR_SHA1_UPDATE			0x00000070
#define		SB_ERROR_SHA1_FINAL			0x00000080
#define		SB_ERROR_VERIFY_RSA_PSS			0x00000090

////////////////////////////////////////////////////////////////////////
//-------------------------------------------
#define		SB20_MAX_EFUSE_DATA_LEN		20

#define		SB_MAX_RSA_KEY		(2048/8)
#define		SB_MAX_SIGN_LEN		SB_MAX_RSA_KEY

#ifdef CONFIG_SECURE_BOOT_V20
#define SB20_HMAC_SHA1_LEN			20
#else
#define SB20_HMAC_SHA1_LEN			32
#endif

#ifdef CONFIG_SECURE_BOOT_V30
#define SB30_MAX_RSA_KEY			(2048/8)
#define SB30_MAX_ECDSA_COORDINATE_BYTELEN	68
#define SB30_HMAC_SHA256_LEN			32
#define SB30_MAX_ECDSA_R_LEN			68
#define SB30_MAX_ECDSA_SIGN_LEN		68
#define SB30_MAX_SIGN_LEN			SB30_MAX_RSA_KEY
#endif

//-------------------------------------------
typedef struct
{
	int			rsa_n_Len;
	unsigned char		rsa_n[SB_MAX_RSA_KEY];
	int			rsa_e_Len;
	unsigned char		rsa_e[4];
} SB_RSAPubKey;

//-------------------------------------------
typedef struct
{
	SB_RSAPubKey		rsaPubKey;
	unsigned char		signedData[SB20_HMAC_SHA1_LEN];
} SB_PubKeyInfo;

//-------------------------------------------
typedef struct
{
	SB_RSAPubKey		stage2PubKey;
	int			code_SignedDataLen;
	unsigned char		code_SignedData[SB_MAX_SIGN_LEN];
	SB_PubKeyInfo		pubKeyInfo;
	unsigned char		func_ptr_BaseAddr[128];
	unsigned char		reservedData[80];
} SB20_CONTEXT;

typedef struct
{
	unsigned int		codesignerversion;
	unsigned int		ap_info;
	unsigned long long	time;
	unsigned int		build_count;
	unsigned char		description[36];
} SB24_INFO;

typedef struct
{
	SB24_INFO		context_info;
	SB_RSAPubKey		stage2PubKey;
	unsigned char		func_ptr_BaseAddr[128];
	unsigned char		reservedData[12];
	SB_PubKeyInfo		pubKeyInfo;
	int			code_SignedDataLen;
	unsigned char		code_SignedData[SB_MAX_SIGN_LEN];
} SB24_CONTEXT;

#if defined(CONFIG_SECURE_BOOT_V30)
typedef struct
{
	int			rsa_n_Len;
	unsigned char		rsa_n[SB30_MAX_RSA_KEY];
	int			rsa_e_Len;
	unsigned char		rsa_e[4];
} SB30_RSAPubKey;

typedef struct
{
	unsigned char		px[SB30_MAX_ECDSA_COORDINATE_BYTELEN];
	unsigned char		py[SB30_MAX_ECDSA_COORDINATE_BYTELEN];
	unsigned char		padding[132];
} SB30_ECDSAPubKey;

typedef union
{
	SB30_RSAPubKey		rsaPubKey;
	SB30_ECDSAPubKey	ecdsaPubKey;

} RSA_ECDSA_PubKey;

typedef struct
{
	RSA_ECDSA_PubKey	rsa_ecdsa_PubKey;
	unsigned char		efuseData[SB30_HMAC_SHA256_LEN];
} SB30_PubKeyInfo;

typedef struct
{
	unsigned int		codesignerversion;
	unsigned int		ap_info;
	unsigned long long	time;
	unsigned int		build_count;
	unsigned int		signing_type;
	unsigned char		description[32];
} SB30_INFO;

typedef struct {
	unsigned char		r[SB30_MAX_ECDSA_R_LEN];
	unsigned char		sign[SB30_MAX_ECDSA_SIGN_LEN];
	unsigned char		padding[120];
} SB30_Ecdsa_Code_SignedData;

typedef union
{
	unsigned char			code_SignedData[SB30_MAX_SIGN_LEN];
	SB30_Ecdsa_Code_SignedData	ecdsa_code_SignedData;

} RSA_ECDSA_SIGN;

typedef struct
{
	SB30_INFO		context_info;
	RSA_ECDSA_PubKey	stage2_PubKey;
	unsigned char		func_ptr_BaseAddr[128];
	unsigned short		major_id;
	unsigned short		sub_id;
	unsigned char		reservedData[8];
	SB30_PubKeyInfo		pubKeyInfo;
	int			code_SignedDataLen;
	RSA_ECDSA_SIGN		rsa_ecdsa_sign;
} SB30_CONTEXT;
#endif

#if defined(CONFIG_SECURE_BOOT_V24)
#define SB20_CONTEXT	SB24_CONTEXT
#elif defined(CONFIG_SECURE_BOOT_V30)
#define SB20_CONTEXT	SB30_CONTEXT
#endif

int check_signature(
		SB20_CONTEXT    *sbContext,
		unsigned char	*data,
		unsigned int	dataLen,
		unsigned char	*signedData,
		unsigned int	signedDataLen );

#ifdef	__cplusplus
}
#endif

#endif /* _UBOOT_SB21_H */
