/*
 * Copyright@ Samsung Electronics Co. LTD
 *
 * This software is proprietary of Samsung Electronics.
 * No part of this software, either material or conceptual may be copied or distributed, transmitted,
 * transcribed, stored in a retrieval system or translated into any human or computer language in any form by any means,
 * electronic, mechanical, manual or otherwise, or disclosed
 * to third parties without the express written permission of Samsung Electronics.
 *
 * Alternatively, this program is free software in case of open source project
 * you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 */

#ifndef __ECT_PARSER_H
#define __ECT_PARSER_H __FILE__

#define BLOCK_AP_THERMAL	"THERMAL"
#define BLOCK_MIF_THERMAL	"MR4"
#define BLOCK_DVFS		"DVFS"
#define BLOCK_ASV		"ASV"
#define BLOCK_TIMING_PARAM	"TIMING"
#define BLOCK_RCC		"RCC"
#define BLOCK_PLL		"PLL"
#define BLOCK_MARGIN		"MARGIN"
#define BLOCK_MINLOCK		"MINLOCK"
#define BLOCK_GEN_PARAM		"GEN"

#define PMIC_VOLTAGE_STEP	(6250)

#define BOOTSAFE_CLEAR		(0)
#define BOOTSAFE_SET		(1)

#define PMU_SYSIP_DAT3		(0x11C8081C)

struct ect_header
{
	char sign[4];
	char version[4];
	unsigned int total_size;
	int num_of_header;
};

struct ect_dvfs_level
{
	unsigned int level;
	int level_en;
};

struct ect_dvfs_domain
{
	char *domain_name;
	unsigned int domain_offset;

	unsigned int max_frequency;
	unsigned int min_frequency;
	int boot_level_idx;
	int resume_level_idx;
	int num_of_clock;
	int num_of_level;
	char **list_clock;
	struct ect_dvfs_level  *list_level;
	unsigned int *list_dvfs_value;
};

struct ect_dvfs_header
{
	int parser_version;
	char version[4];
	int num_of_domain;

	struct ect_dvfs_domain *domain_list;
};

struct ect_pll_frequency
{
	unsigned int frequency;
	unsigned int p;
	unsigned int m;
	unsigned int s;
	unsigned int k;
};

struct ect_pll
{
	char *pll_name;
	unsigned int pll_offset;

	unsigned int type_pll;
	int num_of_frequency;
	struct ect_pll_frequency *frequency_list;
};

struct ect_pll_header
{
	int parser_version;
	char version[4];
	int num_of_pll;

	struct ect_pll *pll_list;
};

struct ect_voltage_table
{
	int table_version;
	int boot_level_idx;
	int resume_level_idx;
	int *level_en;
	unsigned int *voltages;
	unsigned char *voltages_step;
	unsigned int volt_step;
};

struct ect_voltage_domain
{
	char *domain_name;
	unsigned int domain_offset;

	int num_of_group;
	int num_of_level;
	int num_of_table;
	unsigned int *level_list;
	struct ect_voltage_table *table_list;
};

struct ect_voltage_header
{
	int parser_version;
	char version[4];
	int num_of_domain;

	struct ect_voltage_domain *domain_list;
};

struct ect_rcc_table
{
	int table_version;

	unsigned int *rcc;
	unsigned char *rcc_compact;
};

struct ect_rcc_domain
{
	char *domain_name;
	unsigned int domain_offset;

	int num_of_group;
	int num_of_level;
	int num_of_table;
	unsigned int *level_list;
	struct ect_rcc_table *table_list;
};

struct ect_rcc_header
{
	int parser_version;
	char version[4];
	int num_of_domain;

	struct ect_rcc_domain *domain_list;
};

struct ect_mif_thermal_level
{
	int mr4_level;
	unsigned int max_frequency;
	unsigned int min_frequency;
	unsigned int refresh_rate_value;
	unsigned int polling_period;
	unsigned int sw_trip;
};

struct ect_mif_thermal_header
{
	int parser_version;
	char version[4];
	int num_of_level;

	struct ect_mif_thermal_level *level;
};

struct ect_ap_thermal_range
{
	unsigned int lower_bound_temperature;
	unsigned int upper_bound_temperature;
	unsigned int max_frequency;
	unsigned int sw_trip;
	unsigned int flag;
};

struct ect_ap_thermal_function
{
	char *function_name;
	unsigned int function_offset;

	int num_of_range;
	struct ect_ap_thermal_range *range_list;
};

struct ect_ap_thermal_header
{
	int parser_version;
	char version[4];
	int num_of_function;

	struct ect_ap_thermal_function *function_list;
};

struct ect_margin_domain
{
	char *domain_name;
	unsigned int domain_offset;

	int num_of_group;
	int num_of_level;
	unsigned int *offset;
	unsigned char *offset_compact;
	unsigned int volt_step;
};

struct ect_margin_header
{
	int parser_version;
	char version[4];
	int num_of_domain;

	struct ect_margin_domain *domain_list;
};

struct ect_timing_param_size
{
	unsigned int memory_size;
	unsigned int parameter_key;
	unsigned int offset;

	int num_of_timing_param;
	int num_of_level;
	unsigned int *timing_parameter;
};

struct ect_timing_param_header
{
	int parser_version;
	char version[4];
	int num_of_size;

	struct ect_timing_param_size *size_list;
};

struct ect_minlock_frequency
{
	unsigned int main_frequencies;
	unsigned int sub_frequencies;
};

struct ect_minlock_domain
{
	char *domain_name;
	unsigned int domain_offset;

	int num_of_level;
	struct ect_minlock_frequency *level;
};

struct ect_minlock_header
{
	int parser_version;
	char version[4];
	int num_of_domain;

	struct ect_minlock_domain *domain_list;
};

struct ect_gen_param_table
{
	char *table_name;
	unsigned int offset;

	int num_of_col;
	int num_of_row;
	unsigned int *parameter;
};

struct ect_gen_param_header
{
	int parser_version;
	char version[4];
	int num_of_table;

	struct ect_gen_param_table *table_list;
};

struct ect_info
{
	char *block_name;
	int block_name_length;
	int (*parser)(void *address, struct ect_info *info);
	void *block_handle;
};

void ect_init(void);
int ect_parse_binary_header(void);
void *ect_get_block(char *block_name);
struct ect_dvfs_domain *ect_dvfs_get_domain(void *block, char *domain_name);
struct ect_pll *ect_pll_get_pll(void *block, char *pll_name);
struct ect_voltage_domain *ect_asv_get_domain(void *block, char *domain_name);
struct ect_rcc_domain *ect_rcc_get_domain(void *block, char *domain_name);
struct ect_mif_thermal_level *ect_mif_thermal_get_level(void *block, int mr4_level);
struct ect_ap_thermal_function *ect_ap_thermal_get_function(void *block, char *function_name);
struct ect_margin_domain *ect_margin_get_domain(void *block, char *domain_name);
struct ect_timing_param_size *ect_timing_param_get_size(void *block, int size);
struct ect_minlock_domain *ect_minlock_get_domain(void *block, char *domain_name);
struct ect_gen_param_table *ect_gen_param_get_table(void *block, char *table_name);

int ect_check_valid_header(void *default_address, void *test_address);
void ect_init_internal(void *address, void *size);

#endif
