/*
 * Copyright@ Samsung Electronics Co. LTD
 *
 * This software is proprietary of Samsung Electronics.
 * No part of this software, either material or conceptual may be copied or distributed, transmitted,
 * transcribed, stored in a retrieval system or translated into any human or computer language in any form by any means,
 * electronic, mechanical, manual or otherwise, or disclosed
 * to third parties without the express written permission of Samsung Electronics.


 * Alternatively, this program is free software in case of open source projec;
 * you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.

*/

#include "bl2_config.h"
#include "board_config.h"
#include "bl2_smc.h"

unsigned long long exynos_smc(unsigned long long cmd, unsigned long long arg1, unsigned long long arg2, unsigned long long arg3)
{
	register unsigned long long reg0 __asm__("x0") = cmd;
	register unsigned long long reg1 __asm__("x1") = arg1;
	register unsigned long long reg2 __asm__("x2") = arg2;
	register unsigned long long reg3 __asm__("x3") = arg3;

	__asm__ volatile (
		"smc	0\n"
		: "+r"(reg0), "+r"(reg1), "+r"(reg2), "+r"(reg3)

	);

	return reg0;
}

inline unsigned long long exynos_smc_read(unsigned long long cmd, unsigned long long arg1)
{
	register unsigned long long reg0 __asm__("x0") = cmd;
	register unsigned long long reg1 __asm__("x1") = arg1;
	register unsigned long long reg2 __asm__("x2") = 0;

	__asm__ volatile (
		"dsb	sy\n"
		"smc	0\n"
		: "+r"(reg0), "+r"(reg1), "+r"(reg2)

	);

	return reg2;
}

unsigned int load_bl3_image(unsigned int boot_device)
{
	image_info *info_image;

	info_image = (image_info *)CONFIG_SMC_IMAGE_INFORM_BASE;

	info_image->image_base_addr = CONFIG_SYS_TEXT_BASE;
	info_image->size = PART_SIZE_BL3;
	info_image->secure_context_base = SMC_SECURE_CONTEXT_BASE;
	info_image->signature_size = SMC_UBOOT_SIGNATURE_SIZE;

/*	if (boot_device == SDMMC_CH2) {

		info_image->bootdev.sdmmc.image_pos = MOVI_BL3_POS;
		info_image->bootdev.sdmmc.blkcnt =  MOVI_BL3_BLKCNT;
		info_image->bootdev.sdmmc.base_addr = CONFIG_SYS_TEXT_BASE;

	} else*/ if (boot_device == EMMC) {

		info_image->bootdev.emmc.blkcnt =  MOVI_UBOOT_BLKCNT;
		info_image->bootdev.emmc.base_addr = CONFIG_SYS_TEXT_BASE;

	}
#if defined(CONFIG_UBOOT_SECURE_BOOT)
	exynos_smc(SMC_CMD_SET_SIGNATURE_SIZE,
			0, (unsigned int)info_image, 0);
#endif
	return exynos_smc(SMC_CMD_LOAD_UBOOT,
			(unsigned long long)boot_device, (unsigned long long)info_image, (unsigned long long)boot_device);
}

unsigned int coldboot(unsigned int boot_device)
{
	return exynos_smc(SMC_CMD_COLDBOOT, 0, 0, CONFIG_SYS_TEXT_BASE);
}

void warmboot(void)
{
	exynos_smc(SMC_CMD_WARMBOOT, 0, 0, CONFIG_PHY_IRAM_NS_BASE);
}

unsigned int find_second_boot(void)
{
	return exynos_smc_read(SMC_CMD_CHECK_SECOND_BOOT, 0);
}

void emmc_endbootop(void)
{
	exynos_smc(SMC_CMD_EMMC_ENDBOOTOP, 0, 0, 0);
}

void sdmmc_enumerate(void)
{
	exynos_smc(SMC_CMD_SDMMC_ENUMERATE, 0, 0, 0);
}

void set_secure_reg(unsigned long long reg_val, unsigned int num)
{
	exynos_smc(SMC_CMD_SET_SECURE_REG, reg_val, (unsigned long long)num, reg_val);
}

unsigned long long load_ect_head(void)
{
	return exynos_smc(SMC_CMD_LOAD_ECT_HEAD, 0, 0, 0);
}

unsigned long long load_ect_all(void)
{
	return exynos_smc(SMC_CMD_LOAD_ECT_ALL, 0, 0, 0);
}

unsigned long long skip_ect(void)
{
	return exynos_smc(SMC_CMD_SKIP_ECT, 0, 0, 0);
}
