/*
 * Copyright@ Samsung Electronics Co. LTD
 *
 * This software is proprietary of Samsung Electronics.
 * No part of this software, either material or conceptual may be copied or distributed, transmitted,
 * transcribed, stored in a retrieval system or translated into any human or computer language in any form by any means,
 * electronic, mechanical, manual or otherwise, or disclosed
 * to third parties without the express written permission of Samsung Electronics.


 * Alternatively, this program is free software in case of open source project
 * you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.

*/
#define KINGYO
#include <config.h>
#include <version.h>
#include <common.h>
#include <asm/io.h>
#include <asm/arch/exynos7570.h>
#include <asm/arch/smc.h>
#include <asm/arch/pmu.h>

#define CONFIG_BOOTDEVICE_ORDER_ADDR	(CONFIG_PHY_IRAM_BASE + 0x64)

typedef enum {
	BD_NO_DEVICE = 0x0,
	BD_UFS = 0x1,
	BD_EMMC_BOOT = 0x2,
	BD_ERROR = 0x3,
	BD_USB = 0x4,
	BD_SDMMC = 0x5,
	BD_UFS_CARD = 0x6,
	BD_SPI = 0x7,
} boot_device_info_t;

static u32 get_boot_device_info(void)
{
	u32 boot_device_info;

#ifndef CONFIG_SG_BINARY
	if (*(unsigned int *)CONFIG_SYS_SDRAM_BASE != 0xabcdef) {
		/* Running on DRAM by TRACE32 */
		boot_device_info = *(u32 *) CONFIG_BOOTDEVICE_ORDER_ADDR;
	} else {
		boot_device_info = find_second_boot();
	}
#else
	boot_device_info = find_second_boot();
#endif

	return boot_device_info;
}

void read_om(void)
{
	u32 order = 0;
	u32 boot_device = 0;
	u32 boot_device_info = 0;
	u32 boot_device_id;

	boot_device_info = get_boot_device_info();
#ifndef CONFIG_SG_BINARY
	if ((boot_device_info & 0xFF000000) != 0xCB000000) {
		/* abnormal boot */
		while (1) ;
	}
#endif

	order = boot_device_info & 0xF;
	boot_device_id = (boot_device_info >> (4 * order)) & 0xF;

#ifdef KINGYO
	/* Kingyo supports only emmc_4_4 mode */
	boot_device = BOOT_EMMC_4_4;
	/* When pin xom0 is 1, set bootloader mode */
	if (boot_device_id != BD_EMMC_BOOT)
		writel(INFORM_BOOTLOADER, EXYNOS7570_POWER_INFORM4);
#else

	switch (boot_device_id) {
	case BD_UFS:
		boot_device = BOOT_UFS;
		break;
	case BD_EMMC_BOOT:
		boot_device = BOOT_EMMC_4_4;
		break;
	case BD_USB:
		boot_device = BOOT_USB;
		break;
	case BD_SDMMC:
		boot_device = BOOT_MMCSD;
		break;
	case BD_UFS_CARD:
		break;
	case BD_SPI:
		break;
	default:
		break;
	}
#endif
	writel(boot_device, EXYNOS7570_POWER_INFORM3);
}

int is_first_boot(void)
{
	return ((get_boot_device_info() & 0xF) == 1);
}

void set_block_device_info(void)
{
	u32 boot_device = 0;
	u32 boot_device_info = 0;

	boot_device_info = get_boot_device_info();
	switch ((boot_device_info >> 4) & 0xF) {
	case BD_UFS:
		boot_device = BOOT_UFS;
		break;
	case BD_EMMC_BOOT:
		boot_device = BOOT_EMMC_4_4;
		break;
	case BD_USB:
		boot_device = BOOT_USB;
		break;
	case BD_SDMMC:
		boot_device = BOOT_MMCSD;
		break;
	case BD_UFS_CARD:
		break;
	case BD_SPI:
		break;
	default:
		break;
	}

	writel(boot_device, EXYNOS7570_POWER_INFORM3);
}
