/*
 * Copyright@ Samsung Electronics Co. LTD
 *
 * This software is proprietary of Samsung Electronics.
 * No part of this software, either material or conceptual may be copied or distributed, transmitted,
 * transcribed, stored in a retrieval system or translated into any human or computer language in any form by any means,
 * electronic, mechanical, manual or otherwise, or disclosed
 * to third parties without the express written permission of Samsung Electronics.


 * Alternatively, this program is free software in case of open source project
 * you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.

*/

#include <asm-offsets.h>
#include <config.h>
#include <version.h>
#include <linux/linkage.h>
#include <asm/macro.h>
#include "dfd.h"
ENTRY(dfd_entry_dump_gpr)
	/* Check to allow accessing FP/SIMD register */
	mov	x28, x30
	mov	x0, #3 << 20
	msr	cpacr_el1, x0			/* Enable FP/SIMD */

	/* Get the core address of ramdump by each core */
	bl	dfd_get_core_num
	mov	x7, x0
	ldr	x3, =(CONFIG_RAMDUMP_COREREG)
	mov	x2, x0
	ldr	x4, =(DFD_COREREG_OFFSET)
	mul	x2, x2, x4
	add	x3, x3, x2

	mov	x1, x0				/* x1 is core number */
	mov	x0, x3				/* x0 is target address */

	/* x0: target address, x1: core number */
	ldr	x5, =DFD_CPUCL0_DUMP_PC_ADDRESS
1:
	mov	x2, x5
	bl	dfd_dump_gpr
	mov	x2, x5
	bl	dfd_dump_pc

	/* Debug for Cache Flush of dumpGPR : Save initial value 0 */
	mov	w3, #0
	bl	dfd_debug_dump_gpr

	/* Flush cache to get the latest data */
	mov	x0, x7
	bl	dfd_check_core_stat

	/*
	 * x0 = 0, power off, skip cache Flush
	 * x0 = 1, needs Level1(L1=>L2) cache Flush
	 * x0 = 2, needs Level2(L2=>DRAM) cache Flush
	 * x0 = 3, needs Level1, Level2 cache Flush
	 */
	cbz	w0, 2f				/* Skip cache flush */

	cmp	w0, #3				/* Needs all flush */
	b.eq	3f

	/* Debug for Cache Flush of dumpGPR : Save the level of cache flush */
	mov	w3, w0
	bl	dfd_debug_dump_gpr

	sub	w0, w0, #1			/* Sync csselr_el1 bits value */
	mov	x1, #0				/* Only Flush */

	bl	__asm_flush_dcache_level
	mov     x0, #0
	msr     csselr_el1, x0
	dsb     sy
	isb
	b	2f
3:
	/* Debug for Cache Flush of dumpGPR : Save the level of cache flush all */
	mov	w3, w0
	bl	dfd_debug_dump_gpr

	/* Run cache flush all to Level1,2 */
	bl	__asm_flush_dcache_all
2:
	/* Write own bit to inform finishing Dump GPR */
	bl	dfd_get_core_num
	ldr	x1, =CONFIG_RAMDUMP_DUMP_GPR_WAIT
	ldr	w2, [x1]
	mov	w3, #1
	lsl	w3, w3, w0
	orr	w2, w2, w3
	str	w2, [x1]

	/* core0 skip into wfi */
	cbz	w0, 1f

	isb
	dsb	sy
	wfi
1:
	mov	x30, x28
	ret
ENDPROC(dfd_entry_dump_gpr)

ENTRY(dfd_dump_gpr)
	/* Store dump gpr register to dram, x0: target address
	 * The offset is following as kernel's struct pt_regs */

	add	x2, x2, #DFD_DUMP_GPR_OFFSET

	ldp	x3, x4, [x2], #16 /* x0, x1 */
	stp	x3, x4, [x0], #16
	ldp	x3, x4, [x2], #16 /* x2, x3 */
	stp	x3, x4, [x0], #16
	ldp	x3, x4, [x2], #16 /* x4, x5 */
	stp	x3, x4, [x0], #16
	ldp	x3, x4, [x2], #16 /* x6, x7 */
	stp	x3, x4, [x0], #16
	ldp	x3, x4, [x2], #16 /* x8, x19 */
	str	x3, [x0], #88
	str	x4, [x0], #8
	ldp	x3, x4, [x2], #16 /* x20, x21 */
	stp	x3, x4, [x0], #16
	ldp	x3, x4, [x2], #16 /* x22, x23 */
	stp	x3, x4, [x0], #16
	ldp	x3, x4, [x2], #16 /* x24, x25 */
	stp	x3, x4, [x0], #16
	ldp	x3, x4, [x2], #16 /* x26, x27 */
	stp	x3, x4, [x0], #16
	ldp	x3, x4, [x2], #16 /* x28, x29 */
	stp	x3, x4, [x0], #16
	ldp	x3, x4, [x2], #16 /* x30, sp */
	stp	x3, x4, [x0], #16
	ret
ENDPROC(dfd_dump_gpr)

ENTRY(dfd_dump_pc)
	ldr	x3, [x2, x1, lsl #3]
	str	x3, [x0]
	ret
ENDPROC(dfd_dump_pc)

ENTRY(dfd_get_core_num)
	mrs	x1, mpidr_el1
	and	x0, x1, #0xf            /* x0 = cpu id */
	lsr	x1, x1, #8
	and	x1, x1, #0xf            /* x1 = cluster id */
	cmp	x1, #0
	b.eq	1f
	add	x0, x0, #4		/* reorder cpu id */
1:
	ret				/* r0(cpu id) is return */
ENDPROC(dfd_get_core_num)

ENTRY(dfd_debug_dump_gpr)
	/* x3 is save data, and this function uses only x3, x4 registers */
	ldr	x4, =(CONFIG_RAMDUMP_DUMP_GPR_DEBUG)
	add	x4, x4, x7, lsl #2
	str	w3, [x4]
	ret
ENDPROC(dfd_debug_dump_gpr)

ENTRY(dfd_check_core_stat)
	/* x0 is core number */
	ldr	x1, =CONFIG_RAMDUMP_GPR_POWER_STAT
	ldr	w2, [x1, x0, lsl #2]
	lsr	w2, w2, #16
	mov 	x0, x2
	ret
ENDPROC(dfd_check_core_stat)
