/*
 * Copyright@ Samsung Electronics Co. LTD
 *
 * This software is proprietary of Samsung Electronics.
 * No part of this software, either material or conceptual may be copied or distributed, transmitted,
 * transcribed, stored in a retrieval system or translated into any human or computer language in any form by any means,
 * electronic, mechanical, manual or otherwise, or disclosed
 * to third parties without the express written permission of Samsung Electronics.


 * Alternatively, this program is free software in case of open source project
 * you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.

*/

#include <config.h>
#include <common.h>
#include "pmic.h"
#include "../../../drivers/speedy/speedy.h"

/**********************
 * S2MPU06 PMIC
 **********************/
void pmic_enable_peric_dev(void)
{
	unsigned char reg;

	speedy_init();

	speedy_read(S2MPU06_PM_ADDR, S2MPU06_PM_CTRL1, &reg);
	reg &= ~MRSTBEN;
	speedy_write(S2MPU06_PM_ADDR, S2MPU06_PM_CTRL1, reg);

	speedy_read(S2MPU06_PM_ADDR, S2MPU06_PM_CTRL3, &reg);
	reg |= WRSTEN;
	speedy_write(S2MPU06_PM_ADDR, S2MPU06_PM_CTRL3, reg);

	speedy_read(S2MPU06_PM_ADDR, S2MPU06_PM_RTC_BUF, &reg);
	reg |= PERI_32KHZ_EN;
	speedy_write(S2MPU06_PM_ADDR, S2MPU06_PM_RTC_BUF, reg);
}

void pmic_enable_display(void)
{
	unsigned char reg = 0;

	/* LDO13 off when SW reset for codec */
	speedy_read(S2MPU06_PM_ADDR, S2MPU06_PM_LDO13_CTRL, &reg);
	reg &= 0x3F; /* Output off normal */
	speedy_write(S2MPU06_PM_ADDR, S2MPU06_PM_LDO13_CTRL, reg);

	speedy_read(S2MPU06_PM_ADDR, S2MPU06_PM_LDO2_CTRL, &reg);
	reg |= (0x3 << 6); /* Output on normal */
	speedy_write(S2MPU06_PM_ADDR, S2MPU06_PM_LDO2_CTRL, reg);

	reg = (0x3 << 6) | 0x3C; /* Output on normal, 3.3V */
	speedy_write(S2MPU06_PM_ADDR, S2MPU06_PM_LDO19_CTRL, reg);

	reg = (0x3 << 6) | 0x38; /* Output on normal, 2.2V */
	speedy_write(S2MPU06_PM_ADDR, S2MPU06_PM_LDO20_CTRL, reg);
}

/**********************
 * S2MPU06 fuel gauge
 **********************/
static unsigned char model_param[] = {
	0xDF,0x0A,0x3C,0x0A,0xDA,0x09,0x67,0x09,0xFC,0x08,
	0x99,0x08,0x2C,0x08,0xDA,0x07,0x95,0x07,0x4F,0x07,
	0xE3,0x06,0xA3,0x06,0x79,0x06,0x5A,0x06,0x40,0x06,
	0x34,0x06,0x24,0x06,0xF2,0x05,0x96,0x05,0x81,0x05,
	0xD3,0x04,0x43,0x02,0x69,0x08,0x00,0x08,0x97,0x07,
	0x2E,0x07,0xC6,0x06,0x5D,0x06,0xF4,0x05,0x8B,0x05,
	0x22,0x05,0xB9,0x04,0x50,0x04,0xE8,0x03,0x7F,0x03,
	0x16,0x03,0xAD,0x02,0x44,0x02,0x04,0x02,0x72,0x01,
	0x0A,0x01,0xA1,0x00,0x38,0x00,0xFD,0x0F,0xFE,0x29,
	0xDC,0x29,0x8A,0x29,0x4D,0x29,0x28,0x1F,0x7D,0x08,
	0x21,0x21,0x14,0x21,0x0A,0xD9,0xE6,0xF7,0xB0,0xC2,
	0xCA,0xD4,0xD3,0xD4,0x40,0x80,0x26,0x33,0x26,0x40,
	0x26,0x5C,0x26,0x5C,0x26,0x5C,0xFF,0xFF,0x02,0x05,
	0x14,0x0B,0x03,0x12,0x06,0x1C,0x0C,0x1C,0x10,0x1C,
	0x1F,0x1F,0x57,0x57,0x58,0x58,0x59,0x59,0x58,0x5A,
	0x5A,0x5B,0x5C,0x59,0x5C,0x59,0x57,0x5B,0x5A,0x5B,
	0x5A,0x5C,0x9E,0xD7,
};

void fg_s2mpu06_init_param(void)
{
	int i = 0;

	/* 0x0F(rZADJ_CHG2) : battery parameter tuning value */
	speedy_write(S2MPU06_FG_ADDR, 0x0F, 0x30);
	speedy_write(S2MPU06_FG_ADDR, 0x10, 0x28);
	speedy_write(S2MPU06_FG_ADDR, 0x11, 0x0A);

	for (i = 0x34; i <= 0xD7; i++)
		speedy_write(S2MPU06_FG_ADDR, i, model_param[i - 0x34]);
}

void fg_init(void)
{
	unsigned char temp;

	speedy_init();

	/* fuelgauge reset ? */
	speedy_read(S2MPU06_FG_ADDR, 0x1B, &temp);
	printf("S2MPU06_FG 0x1B: 0x%x\n", temp);
	if(temp & 0x10) {
		printf("fuelguage reset !\n");

		speedy_read(S2MPU06_FG_ADDR, 0x1B, &temp);
		temp &= 0xEF;
		speedy_write(S2MPU06_FG_ADDR, 0x1B, temp);

		/* charger disable 0x0F[5:4] = 00 */
		printf("charger disable 0x0F[5:4] = 00\n");
		speedy_read(S2MPU06_CHG_ADDR, 0x0F, &temp);
		temp &= 0xCF;
		speedy_write(S2MPU06_CHG_ADDR, 0x0F, temp);
		speedy_read(S2MPU06_CHG_ADDR, 0x0F, &temp);
		printf("S2MPU06_FG 0x0F: 0x%x\n", temp);

		udelay(100000);

		fg_s2mpu06_init_param();

		/* 0x1A[1:0] = 11 dump done */
		printf("0x1A[1:0] = 11 dump done\n");
		speedy_read(S2MPU06_FG_ADDR, 0x1A, &temp);
		temp &= 0xCC;
		temp |= 0x33;
		speedy_write(S2MPU06_FG_ADDR, 0x1A, temp);
		speedy_read(S2MPU06_FG_ADDR, 0x1A, &temp);
		printf("S2MPU06_FG 0x1A: 0x%x\n", temp);

		udelay(500000);

		/* charger enable 0x0F[5:4] = 01 */
		printf("charger enable 0x0F[5:4] = 01\n");
		speedy_read(S2MPU06_CHG_ADDR, 0x0F, &temp);
		temp &= 0xCF;
		temp |= (0x01 << 4);
		speedy_write(S2MPU06_CHG_ADDR, 0x0F, temp);
		speedy_read(S2MPU06_CHG_ADDR, 0x0F, &temp);
		printf("S2MPU06_FG 0x0F: 0x%x\n", temp);
	}
}
