/*
 *	drivers/video/exynos/panels/auo_h120bln017_lcd_ctrl.c
 */

#include <asm/arch/mipi_dsim.h>
#include "auo_h120bln017_param.h"
#include "lcd_ctrl.h"
#include "../dsim.h"

#define msleep(a) udelay(a * 1000)

#define VIDEO_MODE      1
#define COMMAND_MODE    0

#define ID		0

struct decon_lcd auo_h120bln017_lcd_info = {

	.mode = DECON_MIPI_COMMAND_MODE,

	.decon_hbp = 7,
	.decon_hfp = 1,
	.decon_hsa = 1,

	.decon_vbp = 10,
	.decon_vfp = 2,
	.decon_vsa = 1,

	.dsim_hbp = 1,
	.dsim_hfp = 1,
	.dsim_hsa = 1,

	.dsim_vbp = 10,
	.dsim_vfp = 2,
	.dsim_vsa = 1,

	.xres = 390,
	.yres = 390,

	.width = 32,
	.height = 32,
	.dispif_w = 300,
	.dispif_h = 507,

	/* Mhz */
	.hs_clk = 384,
	.esc_clk = 16,

	.dphy_pms.p = 2,
	.dphy_pms.m = 118,
	.dphy_pms.s = 2,

	.fps = 60,
	.mic_enabled = 0,
	.mic_ratio = MIC_VER_2_0,
	.mic_ver = MIC_VER_1_2,
};

struct decon_lcd *decon_get_lcd_info(void)
{
	return &auo_h120bln017_lcd_info;
}

/*
 * auo_h120bln017 lcd init sequence
 */

void lcd_init(u32 id, struct decon_lcd *lcd)
{
	/* cmd_set_1 */
	while (dsim_wr_data(ID, MIPI_DSI_DCS_LONG_WRITE,
		(unsigned long) cmd_set_1,
		ARRAY_SIZE(cmd_set_1)) == -1)
		dsim_err("********** failed to send cmd_set_1.\n");

	/* cmd_set_2 */
	while (dsim_wr_data(ID, MIPI_DSI_DCS_LONG_WRITE,
		(unsigned long) cmd_set_2,
		ARRAY_SIZE(cmd_set_2)) == -1)
		dsim_err("********** failed to send cmd_set_2.\n");

	/* cmd_set_3 */
	while (dsim_wr_data(ID, MIPI_DSI_DCS_LONG_WRITE,
		(unsigned long) cmd_set_3,
		ARRAY_SIZE(cmd_set_3)) == -1)
		dsim_err("********** failed to send cmd_set_3.\n");

	/* cmd_set_4 */
	while (dsim_wr_data(ID, MIPI_DSI_DCS_LONG_WRITE,
		(unsigned long) cmd_set_4,
		ARRAY_SIZE(cmd_set_4)) == -1)
		dsim_err("********** failed to send cmd_set_4.\n");

	/* cmd_set_5 */
	while (dsim_wr_data(ID, MIPI_DSI_DCS_LONG_WRITE,
		(unsigned long) cmd_set_5,
		ARRAY_SIZE(cmd_set_5)) == -1)
		dsim_err("********** failed to send cmd_set_5.\n");

	/* cmd_set_6 */
	while (dsim_wr_data(ID, MIPI_DSI_DCS_LONG_WRITE,
		(unsigned long) cmd_set_6,
		ARRAY_SIZE(cmd_set_6)) == -1)
		dsim_err("********** failed to send cmd_set_6.\n");

	/* cmd_set_7 */
	while (dsim_wr_data(ID, MIPI_DSI_DCS_LONG_WRITE,
		(unsigned long) cmd_set_7,
		ARRAY_SIZE(cmd_set_7)) == -1)
		dsim_err("********** failed to send cmd_set_7.\n");

	/* cmd_set_8 */
	while (dsim_wr_data(ID, MIPI_DSI_DCS_LONG_WRITE,
		(unsigned long) cmd_set_8,
		ARRAY_SIZE(cmd_set_8)) == -1)
		dsim_err("********** failed to send cmd_set_8.\n");

	/* cmd_set_9 */
	while (dsim_wr_data(ID, MIPI_DSI_DCS_LONG_WRITE,
		(unsigned long) cmd_set_9,
		ARRAY_SIZE(cmd_set_9)) == -1)
		dsim_err("********** failed to send cmd_set_8.\n");

	/* sleep out */
	dsim_wr_data(ID, MIPI_DSI_DCS_SHORT_WRITE,
		0x11, 0);

	/* 150ms delay */
	udelay(150000);

	/* display on */
	dsim_wr_data(ID, MIPI_DSI_DCS_SHORT_WRITE,
		0x29, 0);
}

void lcd_enable(void)
{
}

void lcd_disable(void)
{
	/* display off */
	dsim_wr_data(ID, MIPI_DSI_DCS_SHORT_WRITE,
		0x28, 0);

	/* sleep in */
	dsim_wr_data(ID, MIPI_DSI_DCS_SHORT_WRITE,
		0x10, 0);

	/* 120ms delay */
	udelay(120000);
}
