/* drivers/video/exynos/decon_7580/panels/r63315_lcd_ctrl.c
 *
 * Samsung SoC MIPI LCD CONTROL functions
 *
 * Copyright (c) 2014 Samsung Electronics
 *
 * Jiun Yu, <jiun.yu@samsung.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
*/
#include <asm/arch/mipi_dsim.h>
#include "r63315_param.h"
#include "lcd_ctrl.h"
#include "../dsim.h"

#define GAMMA_PARAM_SIZE 26

#define R63315_VIDEO_VBP		4
#define R63315_VIDEO_VFP		8
#define R63315_VIDEO_VSA		2
#define R63315_VIDEO_HBP		60
#define R63315_VIDEO_HFP		100
#define R63315_VIDEO_HSA		20
#define R63315_HORIZONTAL	1080
#define R63315_VERTICAL		1920

struct decon_lcd r63315_lcd_info = {
	.mode = DECON_VIDEO_MODE,
	.vfp = R63315_VIDEO_VFP,
	.vbp = R63315_VIDEO_VBP,
	.hfp = R63315_VIDEO_HFP,
	.hbp = R63315_VIDEO_HBP,
	.vsa = R63315_VIDEO_VSA,
	.hsa = R63315_VIDEO_HSA,
	.xres = R63315_HORIZONTAL,
	.yres = R63315_VERTICAL,

	/* Maybe, width and height will be removed */
	.width = 70,
	.height = 121,

	/* Mhz */
	.hs_clk = 1100,
	.esc_clk = 20,

	.fps = 60,
	.mic_enabled = 0,
	.mic_ver = 0,
};

struct decon_lcd *decon_get_lcd_info(void)
{
	return &r63315_lcd_info;
}


void lcd_init(u32 id, struct decon_lcd *lcd)
{
	id = 0;

	if (dsim_wr_data(id, MIPI_DSI_DCS_SHORT_WRITE,
				SEQ_SLEEP_OUT[0], 0) < 0)
		dsim_err("fail to send SEQ_SLEEP_OUT command.\n");

	mdelay(200);

	if (dsim_wr_data(id, MIPI_DSI_GENERIC_SHORT_WRITE_2_PARAM,
				SEQ_1[0],
				SEQ_1[1]) < 0)
		dsim_err("fail to send SEQ_1_b0 command.\n");

	if (dsim_wr_data(id, MIPI_DSI_DCS_SHORT_WRITE,
				0, 0) < 0)
		dsim_err("fail to send SEQ_SLEEP_OUT command.\n");

	if (dsim_wr_data(id, MIPI_DSI_DCS_SHORT_WRITE,
				0, 0) < 0)
		dsim_err("fail to send SEQ_SLEEP_OUT command.\n");

	if (dsim_wr_data(id, MIPI_DSI_GENERIC_LONG_WRITE,
				(unsigned long)SEQ_2,
				ARRAY_SIZE(SEQ_2)) < 0)
		dsim_err("fail to send SEQ_2_b3 command.\n");

	if (dsim_wr_data(id, MIPI_DSI_GENERIC_LONG_WRITE,
				(unsigned long)SEQ_3,
				ARRAY_SIZE(SEQ_3)) < 0)
		dsim_err("fail to send SEQ_3 command.\n");

	if (dsim_wr_data(id, MIPI_DSI_GENERIC_SHORT_WRITE_2_PARAM,
				SEQ_4[0],
				SEQ_4[1]) < 0)
		dsim_err("fail to send SEQ_4 command.\n");

	if (dsim_wr_data(id, MIPI_DSI_GENERIC_LONG_WRITE,
				(unsigned long)SEQ_5,
				ARRAY_SIZE(SEQ_5)) < 0)
		dsim_err("fail to send SEQ_5 command.\n");

	if (dsim_wr_data(id, MIPI_DSI_GENERIC_LONG_WRITE,
				(unsigned long)SEQ_6,
				ARRAY_SIZE(SEQ_6)) < 0)
		dsim_err("fail to send SEQ_6 command.\n");

	if (dsim_wr_data(id, MIPI_DSI_GENERIC_LONG_WRITE,
				(unsigned long)SEQ_7,
				ARRAY_SIZE(SEQ_7)) < 0)
		dsim_err("fail to send SEQ_7 command.\n");

	if (dsim_wr_data(id, MIPI_DSI_GENERIC_LONG_WRITE,
				(unsigned long)SEQ_8,
				ARRAY_SIZE(SEQ_8)) < 0)
		dsim_err("fail to send SEQ_8 command.\n");

	if (dsim_wr_data(id, MIPI_DSI_GENERIC_LONG_WRITE,
				(unsigned long)SEQ_9,
				ARRAY_SIZE(SEQ_9)) < 0)
		dsim_err("fail to send SEQ_9 command.\n");

	if (dsim_wr_data(id, MIPI_DSI_GENERIC_LONG_WRITE,
				(unsigned long)SEQ_10,
				ARRAY_SIZE(SEQ_10)) < 0)
		dsim_err("fail to send SEQ_10 command.\n");

	if (dsim_wr_data(id, MIPI_DSI_GENERIC_LONG_WRITE,
				(unsigned long)SEQ_11,
				ARRAY_SIZE(SEQ_11)) < 0)
		dsim_err("fail to send SEQ_11 command.\n");

	if (dsim_wr_data(id, MIPI_DSI_GENERIC_SHORT_WRITE_2_PARAM,
				SEQ_12[0],
				SEQ_12[1]) < 0)
		dsim_err("fail to send SEQ_12 command.\n");

	if (dsim_wr_data(id, MIPI_DSI_GENERIC_LONG_WRITE,
				(unsigned long)SEQ_13,
				ARRAY_SIZE(SEQ_13)) < 0)
		dsim_err("fail to send SEQ_13 command.\n");

	if (dsim_wr_data(id, MIPI_DSI_GENERIC_LONG_WRITE,
				(unsigned long)SEQ_14,
				ARRAY_SIZE(SEQ_14)) < 0)
		dsim_err("fail to send SEQ_14 command.\n");

	if (dsim_wr_data(id, MIPI_DSI_GENERIC_LONG_WRITE,
				(unsigned long)SEQ_15,
				ARRAY_SIZE(SEQ_15)) < 0)
		dsim_err("fail to send SEQ_15 command.\n");

	if (dsim_wr_data(id, MIPI_DSI_GENERIC_SHORT_WRITE_2_PARAM,
				SEQ_16[0],
				SEQ_16[1]) < 0)
		dsim_err("fail to send SEQ_16 command.\n");

	if (dsim_wr_data(id, MIPI_DSI_DCS_SHORT_WRITE,
				SEQ_DISPLAY_ON[0], 0) < 0)
		dsim_err("fail to send SEQ_DISPLAY_ON command.\n");

	mdelay(50);
}

void lcd_enable_exynos(u32 id)
{
	/* This function needs to implement */
}

void lcd_disable(int id)
{
	/* This function needs to implement */
}
