/* drivers/video/exynos/panels/rm69080_lcd_ctrl.c
 *
 * Samsung SoC MIPI LCD CONTROL functions
 *
 * Copyright@ Samsung Electronics Co. LTD
 *
 * Jaeyong Lee <jaeyong2.lee@samsung.com> *
 *
 * This software is proprietary of Samsung Electronics.
 * No part of this software, either material or conceptual may be copied or distributed, transmitted,
 * transcribed, stored in a retrieval system or translated into any human or computer language in any form by any means,
 * electronic, mechanical, manual or otherwise, or disclosed
 * to third parties without the express written permission of Samsung Electronics.


 * Alternatively, this program is free software in case of open source projec;
 * you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.

*/

#include <asm/arch/mipi_dsim.h>
#include "rm69080_param.h"
#include "lcd_ctrl.h"
#include "../dsim.h"

#define msleep(a) udelay(a * 1000)

#define VIDEO_MODE      1
#define COMMAND_MODE    0

#define ID		0

struct decon_lcd rm69080_lcd_info = {
	/*  VIDEO MODE */
	.mode = DECON_MIPI_COMMAND_MODE,

	.decon_hbp = 7,
	.decon_hfp = 1,
	.decon_hsa = 1,

	.decon_vbp = 10,
	.decon_vfp = 2,
	.decon_vsa = 1,

	.dsim_hbp = 1,
	.dsim_hfp = 1,
	.dsim_hsa = 1,

	.dsim_vbp = 10,
	.dsim_vfp = 2,
	.dsim_vsa = 1,

	.xres = 400,
	.yres = 400,

	.width = 35,
	.height = 35,
	.dispif_w = 320,
	.dispif_h = 500,

	/* Mhz */
	.hs_clk = 384,
	.esc_clk = 16,

	.dphy_pms.p = 2,
	.dphy_pms.m = 118,
	.dphy_pms.s = 2,

	.fps = 60,
	.mic_enabled = 0,
	.mic_ratio = MIC_VER_2_0,
	.mic_ver = MIC_VER_1_2,
};

struct decon_lcd *decon_get_lcd_info(void)
{
	return &rm69080_lcd_info;
}

/*
 * rm69080 lcd init sequence
 */

void lcd_init(u32 id, struct decon_lcd *lcd)
{
	/*cmd_mode_page_1_set (to be added)*/
	while (dsim_wr_data(ID, MIPI_DSI_DCS_LONG_WRITE,
		(unsigned long) cmd_mode_page_1_set,
		ARRAY_SIZE(cmd_mode_page_1_set)) == -1)
		dsim_err("failed to send cmd_mode_page_1_set.\n");

	/*cmd_errors_number_read*/
	while (dsim_wr_data(ID, MIPI_DSI_DCS_LONG_WRITE,
		(unsigned long) cmd_errors_number_read,
		ARRAY_SIZE(cmd_errors_number_read)) == -1)
		dsim_err("failed to send cmd_errors_number_read.\n");

	/*cmd_mode_page_2_set */
	while (dsim_wr_data(ID, MIPI_DSI_DCS_LONG_WRITE,
		(unsigned long) cmd_mode_page_2_set,
		ARRAY_SIZE(cmd_mode_page_2_set)) == -1)
		dsim_err("failed to send cmd_mode_page_2_set.\n");

	/*cmd_07_set */
	while (dsim_wr_data(ID, MIPI_DSI_DCS_LONG_WRITE,
		(unsigned long) cmd_07_set,
		ARRAY_SIZE(cmd_07_set)) == -1)
		dsim_err("failed to send cmd_07_set.\n");

	/*cmd_mode_page_3_set */
	while (dsim_wr_data(ID, MIPI_DSI_DCS_LONG_WRITE,
		(unsigned long) cmd_mode_page_3_set,
		ARRAY_SIZE(cmd_mode_page_3_set)) == -1)
		dsim_err("failed to send cmd_mode_page_3_set.\n");

	/*cmd_1C_set */
	while (dsim_wr_data(ID, MIPI_DSI_DCS_LONG_WRITE,
		(unsigned long) cmd_1C_set,
		ARRAY_SIZE(cmd_1C_set)) == -1)
		dsim_err("failed to send cmd_1C_set.\n");

	/*cmd_mode_page_4_set */
	while (dsim_wr_data(ID, MIPI_DSI_DCS_LONG_WRITE,
		(unsigned long) cmd_mode_page_4_set,
		ARRAY_SIZE(cmd_mode_page_4_set)) == -1)
		dsim_err("failed to send cmd_mode_page_4_set.\n");

	/*cmd_TE_line_on */
	while (dsim_wr_data(ID, MIPI_DSI_DCS_LONG_WRITE,
		(unsigned long) cmd_TE_line_on,
		ARRAY_SIZE(cmd_TE_line_on)) == -1)
		dsim_err("failed to cmd_TE_line_on.\n");

	/*idle mode */
	dsim_wr_data(ID, MIPI_DSI_DCS_SHORT_WRITE,
		0x39, 0);

	/* sleep out */
	dsim_wr_data(ID, MIPI_DSI_DCS_SHORT_WRITE,
		0x11, 0);

	/* 300ms delay */
	msleep(300);

	/* display on */
	dsim_wr_data(ID, MIPI_DSI_DCS_SHORT_WRITE,
		0x29, 0);
}


void lcd_enable(void)
{
}

void lcd_disable(void)
{
	/* display off */
	dsim_wr_data(ID, MIPI_DSI_DCS_SHORT_WRITE,
		0x28, 0);

	/* sleep in */
	dsim_wr_data(ID, MIPI_DSI_DCS_SHORT_WRITE,
		0x10, 0);

	/* 120ms delay */
	msleep(120);
}

