/* drivers/video/exynos/panels/s6e3ha2_lcd_ctrl.c
 *
 * Samsung SoC MIPI LCD CONTROL functions
 *
 * Copyright@ Samsung Electronics Co. LTD
 *
 * Jiun Yu, <jiun.yu@samsung.com> *
 *
 * This software is proprietary of Samsung Electronics.
 * No part of this software, either material or conceptual may be copied or distributed, transmitted,
 * transcribed, stored in a retrieval system or translated into any human or computer language in any form by any means,
 * electronic, mechanical, manual or otherwise, or disclosed
 * to third parties without the express written permission of Samsung Electronics.


 * Alternatively, this program is free software in case of open source projec;
 * you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.

*/


#include <asm/arch/mipi_dsim.h>
#include "s6e3ha2_param.h"
#include "lcd_ctrl.h"
#include "../dsim.h"

#define msleep(a) udelay(a * 1000)

/* Porch values. It depends on command or video mode */
#define S6E3HA2_CMD_VBP	15
#define S6E3HA2_CMD_VFP	3
#define S6E3HA2_CMD_VSA	1
#define S6E3HA2_CMD_HBP	2
#define S6E3HA2_CMD_HFP	2
#define S6E3HA2_CMD_HSA	2

/* These need to define */
#define S6E3HA2_VIDEO_VBP	15
#define S6E3HA2_VIDEO_VFP	1
#define S6E3HA2_VIDEO_VSA	1
#define S6E3HA2_VIDEO_HBP	20
#define S6E3HA2_VIDEO_HFP	20
#define S6E3HA2_VIDEO_HSA	20

#define S6E3HA2_HORIZONTAL	1440
#define S6E3HA2_VERTICAL	2560

#define CONFIG_FB_I80_COMMAND_MODE

struct decon_lcd s6e3ha2_lcd_info = {
#ifdef CONFIG_FB_I80_COMMAND_MODE
	.mode = DECON_MIPI_COMMAND_MODE,
	.vfp = S6E3HA2_CMD_VFP,
	.vbp = S6E3HA2_CMD_VBP,
	.hfp = S6E3HA2_CMD_HFP,
	.hbp = S6E3HA2_CMD_HBP,
	.vsa = S6E3HA2_CMD_VSA,
	.hsa = S6E3HA2_CMD_HSA,
#else
	.mode = DECON_VIDEO_MODE,
	.vfp = S6E3HA2_VIDEO_VFP,
	.vbp = S6E3HA2_VIDEO_VBP,
	.hfp = S6E3HA2_VIDEO_HFP,
	.hbp = S6E3HA2_VIDEO_HBP,
	.vsa = S6E3HA2_VIDEO_VSA,
	.hsa = S6E3HA2_VIDEO_HSA,
#endif
	.xres = S6E3HA2_HORIZONTAL,
	.yres = S6E3HA2_VERTICAL,

	/* Maybe, width and height will be removed */
	.width = 80,
	.height = 120,

	/* Mhz */
	.hs_clk = 1100,
	.esc_clk = 20,

	.fps = 60,
	.mic_enabled = 0,
	.mic_ratio = MIC_COMP_RATIO_1_2,
	.mic_ver = MIC_VER_1_2,
	.dphy_pms.p = 3,
	.dphy_pms.m = 127,
	.dphy_pms.s = 0
};

struct decon_lcd *decon_get_lcd_info(void)
{
	return &s6e3ha2_lcd_info;
}


void lcd_init(u32 id, struct decon_lcd *lcd)
{
	if (dsim_wr_data(id, MIPI_DSI_DCS_LONG_WRITE, (unsigned long)SEQ_TEST_KEY_ON_F0,
				ARRAY_SIZE(SEQ_TEST_KEY_ON_F0)) < 0)
		dsim_err("fail to write KEY_ON init command.\n");

	if (dsim_wr_data(id, MIPI_DSI_DCS_LONG_WRITE, (unsigned long)SEQ_TEST_KEY_ON_FC,
				ARRAY_SIZE(SEQ_TEST_KEY_ON_FC)) < 0)
		dsim_err("fail to write FC init command.\n");

	if (dsim_wr_data(id, MIPI_DSI_DCS_SHORT_WRITE, (unsigned long)SEQ_SLEEP_OUT[0], 0) < 0)
		dsim_err("fail to write SLEEP_OUT init command.\n");

	msleep(10);

	if (dsim_wr_data(id, MIPI_DSI_DCS_SHORT_WRITE_PARAM, (unsigned long)SEQ_DUAL_DSI[0],
				(unsigned long)SEQ_DUAL_DSI[1]) < 0)
		dsim_err("fail to write DUAL init command.\n");
	msleep(120);

	if (dsim_wr_data(id, MIPI_DSI_DCS_SHORT_WRITE, (unsigned long)SEQ_TE_ON[0], 0) < 0)
		dsim_err("fail to write TE_START_SETTING command.\n");

	if (dsim_wr_data(id, MIPI_DSI_DCS_LONG_WRITE, (unsigned long)SEQ_GAMMA_CONDITION_SET,
				ARRAY_SIZE(SEQ_GAMMA_CONDITION_SET)) < 0)
		dsim_err("fail to write GAMMA_CONDITION_SET init command.\n");

	if (dsim_wr_data(id, MIPI_DSI_DCS_LONG_WRITE, (unsigned long)SEQ_GAMMA_UPDATE,
				ARRAY_SIZE(SEQ_GAMMA_UPDATE)) < 0)
		dsim_err("fail to write GAMMA_UPDATE init command.\n");

	if (dsim_wr_data(id, MIPI_DSI_DCS_LONG_WRITE, (unsigned long)SEQ_TEST_KEY_OFF_F0,
				ARRAY_SIZE(SEQ_TEST_KEY_OFF_F0)) < 0)
		dsim_err("fail to write KEY_OFF init command.\n");

	/* Added 120ms delay before SEQ_DISPLAY_ON */
	msleep(120);

}

void lcd_enable_exynos(u32 id)
{
	if (dsim_wr_data(id, MIPI_DSI_DCS_SHORT_WRITE, (unsigned long)SEQ_DISPLAY_ON[0], 0) < 0)
		dsim_err("fail to write DISPLAY_ON command.\n");
}

void lcd_disable(int id)
{
}

int lcd_gamma_ctrl(u32 id, u32 backlightlevel)
{
	return 0;
}

int lcd_gamma_update(int id)
{
	return 0;
}
