/* drivers/video/exynos/panels/s6e3ha2k_lcd_ctrl.c
 *
 * Samsung SoC MIPI LCD CONTROL functions
 *
 * Copyright@ Samsung Electronics Co. LTD
 *
 * Jiun Yu, <jiun.yu@samsung.com> *
 *
 * This software is proprietary of Samsung Electronics.
 * No part of this software, either material or conceptual may be copied or distributed, transmitted,
 * transcribed, stored in a retrieval system or translated into any human or computer language in any form by any means,
 * electronic, mechanical, manual or otherwise, or disclosed
 * to third parties without the express written permission of Samsung Electronics.


 * Alternatively, this program is free software in case of open source projec;
 * you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.

*/


#include <asm/arch/mipi_dsim.h>
#include "s6e3ha2k_param.h"
#include "lcd_ctrl.h"
#include "../dsim.h"

#define msleep(a) udelay(a * 1000)

/* Porch values. It depends on command or video mode */
#define S6E3HA2K_CMD_VBP	15
#define S6E3HA2K_CMD_VFP	3
#define S6E3HA2K_CMD_VSA	1
#define S6E3HA2K_CMD_HBP	2
#define S6E3HA2K_CMD_HFP	2
#define S6E3HA2K_CMD_HSA	2

/* These need to define */
#define S6E3HA2K_VIDEO_VBP	15
#define S6E3HA2K_VIDEO_VFP	1
#define S6E3HA2K_VIDEO_VSA	1
#define S6E3HA2K_VIDEO_HBP	20
#define S6E3HA2K_VIDEO_HFP	20
#define S6E3HA2K_VIDEO_HSA	20

#define S6E3HA2K_HORIZONTAL	1440
#define S6E3HA2K_VERTICAL	2560

#define CONFIG_FB_I80_COMMAND_MODE

struct decon_lcd s6e3ha2k_lcd_info = {
#ifdef CONFIG_FB_I80_COMMAND_MODE
	.mode = DECON_MIPI_COMMAND_MODE,
	.vfp = S6E3HA2K_CMD_VFP,
	.vbp = S6E3HA2K_CMD_VBP,
	.hfp = S6E3HA2K_CMD_HFP,
	.hbp = S6E3HA2K_CMD_HBP,
	.vsa = S6E3HA2K_CMD_VSA,
	.hsa = S6E3HA2K_CMD_HSA,
#else
	.mode = DECON_VIDEO_MODE,
	.vfp = S6E3HA2K_VIDEO_VFP,
	.vbp = S6E3HA2K_VIDEO_VBP,
	.hfp = S6E3HA2K_VIDEO_HFP,
	.hbp = S6E3HA2K_VIDEO_HBP,
	.vsa = S6E3HA2K_VIDEO_VSA,
	.hsa = S6E3HA2K_VIDEO_HSA,
#endif
	.xres = S6E3HA2K_HORIZONTAL,
	.yres = S6E3HA2K_VERTICAL,

	/* Maybe, width and height will be removed */
	.width = 80,
	.height = 120,

	/* Mhz */
	.hs_clk = 1100,
	.esc_clk = 20,

	.fps = 60,
	.mic_enabled = 1,
	.mic_ratio = MIC_COMP_RATIO_1_2,
	.mic_ver = MIC_VER_1_2,
	.dphy_pms.p = 3,
	.dphy_pms.m = 127,
	.dphy_pms.s = 0
};

struct decon_lcd *decon_get_lcd_info(void)
{
	return &s6e3ha2k_lcd_info;
}


void lcd_init(u32 id, struct decon_lcd *lcd)
{
	if (dsim_wr_data(id, MIPI_DSI_DCS_LONG_WRITE, (unsigned long)SEQ_TEST_KEY_ON_F0,
				ARRAY_SIZE(SEQ_TEST_KEY_ON_F0)) < 0)
		dsim_err("fail to write KEY_ON init command.\n");

	if (dsim_wr_data(id, MIPI_DSI_DCS_LONG_WRITE, (unsigned long)SEQ_REG_F2,
				ARRAY_SIZE(SEQ_REG_F2)) < 0)
		dsim_err("fail to write F2 init command.\n");

	if (s6e3ha2k_lcd_info.mic_enabled)
		if (dsim_wr_data(id, MIPI_DSI_DCS_LONG_WRITE, (unsigned long)SEQ_REG_F9,
					ARRAY_SIZE(SEQ_REG_F9)) < 0)
			dsim_err("fail to write F9 init command.\n");

	if (dsim_wr_data(id, MIPI_DSI_DCS_SHORT_WRITE, (unsigned long)SEQ_SLEEP_OUT[0], 0) < 0)
		dsim_err("fail to write SLEEP_OUT init command.\n");

	msleep(10);

	if (dsim_wr_data(id, MIPI_DSI_DCS_LONG_WRITE, (unsigned long)SEQ_TEST_KEY_ON_F0,
				ARRAY_SIZE(SEQ_TEST_KEY_ON_F0)) < 0)
		dsim_err("fail to write KEY_ON init command.\n");

	/* TE rising time change : 10 line earlier */
	if (dsim_wr_data(id, MIPI_DSI_DCS_LONG_WRITE, (unsigned long)SEQ_TE_START_SETTING,
				ARRAY_SIZE(SEQ_TE_START_SETTING)) < 0)
		dsim_err("fail to write TE_START_SETTING command.\n");

	if (dsim_wr_data(id, MIPI_DSI_DCS_LONG_WRITE, (unsigned long)SEQ_REG_F2,
				ARRAY_SIZE(SEQ_REG_F2)) < 0)
		dsim_err("fail to write F2 init command.\n");

	if (dsim_wr_data(id, MIPI_DSI_DCS_SHORT_WRITE, SEQ_TE_ON[0], 0) < 0)
		dsim_err("fail to write TE_on init command.\n");

	if (dsim_wr_data(id, MIPI_DSI_DCS_LONG_WRITE, (unsigned long)SEQ_TOUCH_HSYNC,
				ARRAY_SIZE(SEQ_TOUCH_HSYNC)) < 0)
		dsim_err("fail to write TOUCH_HSYNC init command.\n");

	if (dsim_wr_data(id, MIPI_DSI_DCS_LONG_WRITE, (unsigned long)SEQ_PENTILE_CONTROL,
				ARRAY_SIZE(SEQ_PENTILE_CONTROL)) < 0)
		dsim_err("fail to write PENTILE_CONTROL init command.\n");

	if (dsim_wr_data(id, MIPI_DSI_DCS_LONG_WRITE, (unsigned long)SEQ_COLUMN_ADDRESS,
				ARRAY_SIZE(SEQ_COLUMN_ADDRESS)) < 0)
		dsim_err("fail to write COLUMN_ADDRESS init command.\n");

	if (dsim_wr_data(id, MIPI_DSI_DCS_LONG_WRITE, (unsigned long)SEQ_GAMMA_CONDITION_SET,
				ARRAY_SIZE(SEQ_GAMMA_CONDITION_SET)) < 0)
		dsim_err("fail to write GAMMA_CONDITION_SET init command.\n");

	if (dsim_wr_data(id, MIPI_DSI_DCS_LONG_WRITE, (unsigned long)SEQ_AID_SET,
				ARRAY_SIZE(SEQ_AID_SET)) < 0)
		dsim_err("fail to write AID_SET init command.\n");

	if (dsim_wr_data(id, MIPI_DSI_DCS_LONG_WRITE, (unsigned long)SEQ_ELVSS_SET,
				ARRAY_SIZE(SEQ_ELVSS_SET)) < 0)
		dsim_err("fail to write ELVSS_SET init command.\n");

	if (dsim_wr_data(id, MIPI_DSI_DCS_LONG_WRITE, (unsigned long)SEQ_GAMMA_UPDATE,
				ARRAY_SIZE(SEQ_GAMMA_UPDATE)) < 0)
		dsim_err("fail to write GAMMA_UPDATE init command.\n");

	if (dsim_wr_data(id, MIPI_DSI_DCS_LONG_WRITE, (unsigned long)SEQ_ACL_OFF,
				ARRAY_SIZE(SEQ_ACL_OFF)) < 0)
		dsim_err("fail to write ACL_OFF init command.\n");

	if (dsim_wr_data(id, MIPI_DSI_DCS_LONG_WRITE, (unsigned long)SEQ_ACL_OPR,
				ARRAY_SIZE(SEQ_ACL_OPR)) < 0)
		dsim_err("fail to write ACL_OPR init command.\n");

	if (dsim_wr_data(id, MIPI_DSI_DCS_LONG_WRITE, (unsigned long)SEQ_HBM_OFF,
				ARRAY_SIZE(SEQ_HBM_OFF)) < 0)
		dsim_err("fail to write HBM_OFF init command.\n");

	if (dsim_wr_data(id, MIPI_DSI_DCS_LONG_WRITE, (unsigned long)SEQ_TSET_GLOBAL,
				ARRAY_SIZE(SEQ_TSET_GLOBAL)) < 0)
		dsim_err("fail to write TSET_GLOBAL init command.\n");

	if (dsim_wr_data(id, MIPI_DSI_DCS_LONG_WRITE, (unsigned long)SEQ_TSET,
				ARRAY_SIZE(SEQ_TSET)) < 0)
		dsim_err("fail to write TSET init command.\n");

	if (dsim_wr_data(id, MIPI_DSI_DCS_LONG_WRITE, (unsigned long)SEQ_TEST_KEY_OFF_F0,
				ARRAY_SIZE(SEQ_TEST_KEY_OFF_F0)) < 0)
		dsim_err("fail to write KEY_OFF init command.\n");

	/* Added 120ms delay before SEQ_DISPLAY_ON */
	msleep(120);

}

void lcd_enable_exynos(u32 id)
{
	if (dsim_wr_data(id, MIPI_DSI_DCS_SHORT_WRITE, (unsigned long)SEQ_DISPLAY_ON[0], 0) < 0)
		dsim_err("fail to write DISPLAY_ON command.\n");
}

void lcd_disable(int id)
{
}

int lcd_gamma_ctrl(u32 id, u32 backlightlevel)
{
	return 0;
}

int lcd_gamma_update(int id)
{
	return 0;
}
